
/************************************************************************/
/*									*/
/*			CSys Labs Incorporated				*/	
/*			    Eugene Engelgau  				*/
/*									*/
/************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <time.h>
#include <iostream.h>
#include <csysdefs.h>
#include <tviopp.h>

#define SERIAL

int GetRandom(int LLimit,int HiLimit,int Offset);

/* Cord identifier */
typedef struct sdefCord CORD;
struct sdefCord 		/* Coordinate structure */
{
        int X;
	int Y;
};
class Point : CORD
{
	int LX;			/* Left X boundary */
	int RX;			/* Left X boundary */
	int UY;			/* Upper Y boundary */
	int LY;			/* Lower Y boundary */
    public:
	int DX;			/* Delta Y */
	int DY;
	STRING * pName;		/* POINT name */
	Point(int InitLX,int InitRX,int InitUY,int InitLY,STRING * InitpName);/* Inits new point */
	int GetX();		/* Returns X */
	int GetY();		/* Returns X */
	void MovePoint();	/* Randomly moves point */
	bool Touching();	/* Return 1 if point is touching */
	void SetDXDY();		/* Sets new DX and DY */
};
class Pair
{
	int DXL;		/* Delta X Limit */
	int DYL;		/* Delta Y Limit */
    public:
	Point A;		/* A pair has 2 points */
	Point B;
	Pair(int InitLX,int InitRX,int InitUY,int InitLY,int LX,int LY);/* Inits new point */
	void LimitXY();		/* Limits X/Y range */
};
class Block: public Pair
{
    protected:
	tvm * pTVM;
    public:
	Block(tvm *,int InitLX,int InitRX,int InitUY,int InitLY);/* Inits new point */
	void Draw();
};
class Box: public Pair
{
    protected:
	tvm * pTVM;
	int Thickness;
    public:
	Box(tvm *,int InitLX,int InitRX,int InitUY,int InitLY,int Th);/* Inits new point */
	void Draw();
};

/* POINT constructor */
Point::Point(int InitLX,int InitRX,int InitUY,int InitLY,STRING * InitpName)
{
    LX = InitLX;		/* Init borders */
    RX = InitRX;
    UY = InitUY;
    LY = InitLY;
    pName = InitpName;		/* Init point name */
    X = GetRandom(LX,RX,0);	/* Init XY */
    Y = GetRandom(UY,LY,0);
    Point::SetDXDY();			/* Set DX/DY */
}
bool Point::Touching()
{
    bool bReturn = 0;
    if(X < LX)
    {
	X = LX;
	bReturn = 1;
    }
    else if(X > RX)
    {
        X = RX;
	bReturn = 1;
    }
    if(Y < UY)
    {
        Y = UY;
	bReturn = 1;
    }
    else if(Y > LY)
    {
	Y = LY;
	bReturn = 1;
    }
    return(bReturn);
}
void Point::SetDXDY()
{
    DX = GetRandom(0,8,4);
    do
    {
	DY = GetRandom(0,8,4);
    }while(!DY&&!DX);
}
void Point::MovePoint()
{
    X += DX;
    Y += DY;
    if(Point::Touching())
    {
	Point::SetDXDY();
    }
}
int Point::GetX()
{
    return(X);
}
int Point::GetY()
{
    return(Y);
}
/* Pair constructor */
Pair::Pair(int InitLX,int InitRX,int InitUY,int InitLY,int LX,int LY):/* Inits new point */
	A(InitLX,InitRX,InitUY,InitLY,NULLP),
	B(InitLX,InitRX,InitUY,InitLY,NULLP)
{
	DXL = LX;
	DYL = LY;
};
void Pair::LimitXY()
{
    if(abs(A.GetX() - B.GetX()) > DXL)
    {
	if(A.GetX() < B.GetX())
	{
	    A.DX = GetRandom(1,4,0);
	    B.DX = GetRandom(1,4,-4);
	}
	else
	{
	    A.DX = GetRandom(1,4,-4);
	    B.DX = GetRandom(1,4,0);
	}
    }
    if(abs(A.GetY() - B.GetY()) > DYL)
    {
	if(A.GetY() < B.GetY())
	{
	    A.DY = GetRandom(1,4,0);
	    B.DY = GetRandom(1,4,-4);
	}
	else
	{
	    A.DY = GetRandom(1,4,-4);
	    B.DY = GetRandom(1,4,0);
	}
    }
}
Block::Block(tvm * pT,int InitLX,int InitRX,int InitUY,int InitLY):/* Inits new block */
	Pair(InitLX,InitRX,InitUY,InitLY,50,50)
{
    pTVM = pT;
};
Box::Box(tvm * pT,int InitLX,int InitRX,int InitUY,int InitLY,int Th):/* Inits new block */
	Pair(InitLX,InitRX,InitUY,InitLY,50,50)
{
    pTVM = pT;
    Thickness = Th;
};
void Block::Draw()
{
    pTVM->DrawBlock(A.GetX(),A.GetY(),B.GetX(),B.GetY(),1);
    A.MovePoint();		/* Move A Point */
    B.MovePoint();		/* Move Point B */
}
void Box::Draw()
{
    pTVM->DrawBox(A.GetX(),A.GetY(),B.GetX(),B.GetY(),Thickness * 2);
    A.MovePoint();		/* Move A Point */
    B.MovePoint();		/* Move Point B */
}

void main(int argc,char * argv[])
{
    int MaxX;
    bool bSerial = 0;
    bool bMode = 0;		/* Stop = 0 so is running */
    uchar KeyCode;		/* Key code buffer */
    TVM * pTVM;

    if(argc > 1)
    {
	if(!strcmp(argv[1],"-s"))	// See if -s, serial flag
	{
	    bSerial = 1;
	}
	else if(strcmp(argv[1],"-p"))   // See if not parallel flag
	{
	    cout << "ERROR:\tIllegal options, Legal options are:\n";
	    cout << "\t-p\tParallel mode (Default)\n";
	    cout << "\t-s\tSerial mode\n";
	    exit(0);
	}
    }
    if(bSerial)
    {
	pTVM = new tvms;
	if(((TVMS *)pTVM)->StartUp(COM1,IRQDEF,B38400,"XONXOFF"))// Start serial module
	{
	    exit(0);		/* No contact so exit */
	}
	((TVMS *)pTVM)->SetLowThreshold(300);
	((TVMS *)pTVM)->SetHighThreshold(600);
    }
    else
    {
	pTVM = new tvmp;
	if(((TVMP *)pTVM)->StartUp())	/* Establish contact with TVM */
	{
	    exit(0);		/* No contact so exit */
	}
    }
    MaxX = pTVM->MaxX;
    pTVM->SetEL(1);
    pTVM->CursorAttribute(0);		/* Turn cursor OFF */
    pTVM->LoadButtonBuffer("EXIT");	/* Send EXIT string to internal buffer */
    pTVM->PlaceButton(10,0,0);	/* Places EXIT button top left, code is zero */
    pTVM->LoadButtonBuffer("Mode2");	/* Load test */
    pTVM->PlaceButton(20,0,2);	/* Places Mode2 in lower left */
    Point P1(46,239,0,MaxX,"W");	/* Init points */
    Point P2(46,239,0,MaxX,"X");	/* Init points */
    Point P3(46,239,0,MaxX,"Y");	/* Init points */
    Point P4(46,239,0,MaxX,"Z");	/* Init points */
    Box B1(pTVM,47,239,0,MaxX,1);	/* Instantiate Box's */
    Box B2(pTVM,47,239,0,MaxX,2);
    Box B3(pTVM,47,239,0,MaxX,3);
    Box B4(pTVM,47,239,0,MaxX,4);
    Block BL1(pTVM,46,239,0,MaxX);	/* Instantiate Blocks */
    while(1)			/* Loop till exit */
    {
	if(pTVM->CheckKEYPRESS())	/* See if a key is pressed */
	{
	    KeyCode = pTVM->ReadKeyCode();	/* Get key code */
	    if(KeyCode == 10)	/* Leaving progeam */
	    {
		pTVM->DeleteAllButtons();	/* Delete buttons */
		pTVM->BlankDisplay();		/* Clear display */
		pTVM->SetXY(0,0);	 	/* Set cursor upper left */
		pTVM->InputString("Bye Bye!!!");	/* Say good byes */
		pTVM->Refresh();
		exit(0);		/* Leave program */
	    }
	    else if(KeyCode == 20)
	    {
		pTVM->DeleteButton(20);	/* Remove Mode1/Mode2 button */
		if(!bMode)		/* See if running */
		{
		    bMode = 1;		/* Set stop flag to stop */
		    pTVM->LoadButtonBuffer("Mode1");	/* Input Mode1 string */
		}
		else
		{
		    bMode = 0;		/* Start running */
		    pTVM->LoadButtonBuffer("Mode2");	/* Input Mode2 string */
		}
		pTVM->PlaceButton(20,0,2);	/* Places button in lower left */
	    }
	}
	if(!bMode)
	{
	    pTVM->DrawBlock(46,0,239,MaxX,0);	/* Blank display area */
	    P1.MovePoint();
	    P2.MovePoint();
	    P3.MovePoint();
	    P4.MovePoint();
	    BL1.Draw();
	    B4.Draw();
	    B3.Draw();
	    B2.Draw();
	    B1.Draw();
	    pTVM->DrawVector(P1.GetX(),P1.GetY(),P2.GetX(),P2.GetY(),1);
	    pTVM->DrawVector(P2.GetX(),P2.GetY(),P3.GetX(),P3.GetY(),1);
	    pTVM->DrawVector(P3.GetX(),P3.GetY(),P4.GetX(),P4.GetY(),1);
	    pTVM->DrawVector(P4.GetX(),P4.GetY(),P1.GetX(),P1.GetY(),1);
	    pTVM->Refresh();
	}
	else
	{
	    switch(GetRandom(1,2,0))
	    {
		case 0:
		    Block BL10(pTVM,46,239,0,MaxX);
		    BL10.Draw();
		    break;
		case 1:
		    Box B10(pTVM,47,239,0,MaxX,GetRandom(1,3,0));
		    B10.Draw();
		    break;
		case 2:
		    Point P10(46,239,0,MaxX,NULLP);	/* Init points */
		    Point P11(46,239,0,MaxX,NULLP);	/* Init points */
		    Point P12(46,239,0,MaxX,NULLP);	/* Init points */
		    pTVM->DrawVector(P10.GetX(),P10.GetY(),P11.GetX(),P11.GetY(),1);
		    pTVM->DrawVector(P11.GetX(),P11.GetY(),P12.GetX(),P12.GetY(),1);
		    pTVM->DrawVector(P12.GetX(),P12.GetY(),P10.GetX(),P10.GetY(),1);
		    break;
	    }
	    pTVM->Refresh();
	}
    }
}
/* Get random returns a random number between LLimit and HiLimit.  It is used by
various "act" routines */
int GetRandom(int LLimit,int HiLimit,int Offset)
{
    int i= 0xffff;
    while(i < LLimit)
    {
	i = ((int)rand() % (HiLimit + 1));
    }
    return(i - Offset);
}
