
/************************************************************************/
/*									*/
/*			CSys Labs Incorporated				*/
/*			    Eugene Engelgau  				*/
/*									*/
/************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <csysdefs.h>
#include <tvios.h>
#include <time.h>

/* This is a clock example.  It uses the serial interface as well. */

typedef struct sdefCords CORDS;
struct sdefCords
{
    int X;
    int Y;
};

extern bool Verbose;

void MakeClockFace();
void srClock();
void MoveCord(CORDS *,CORDS *);
void AddCord(CORDS *,CORDS *,CORDS *);
void Pol2Rec(float Angle,float Length, CORDS * pCord);
void Hours(int Angle,bool bType);
void Minutes(int Angle,bool bType);
void Seconds(int Angle,bool bType);

void main()
{
    if(StartUp(COM1,IRQDEF,B38400,"RTSCTS"))/* Establish contact with TVM */
    {
	exit(0);		/* No contact so exit */
    }
    printf("Press any key to quit\n");
    MakeClockFace();
    while(!kbhit())
    {
	srClock();
    }
}

void MoveCord(CORDS * pDest,CORDS * pSrc)
{
    pDest->X = pSrc->X;
    pDest->Y = pSrc->Y;
}
void AddCord(CORDS * pDest,CORDS * pA,CORDS * pB)
{
    pDest->X = pA->X + pB->X;
    pDest->Y = pA->Y + pB->Y;
}

void Pol2Rec(float Angle,float Length, CORDS * pCord)
{
    float X,Y;
    Angle = (180 - Angle) / 57.2958;
    pCord->X = sin(Angle) * Length;
    pCord->Y = cos(Angle) * Length;
}

void DrawCircle(CORDS * pCenter,int Diameter,int DegPerSeg)
{
    CORDS Point,OldPoint;
    int Angle;
    Pol2Rec(0,Diameter,&OldPoint);
    AddCord(&OldPoint,&OldPoint,pCenter);
    for(Angle = DegPerSeg;Angle < 361;Angle+=DegPerSeg)
    {
	Pol2Rec(Angle,Diameter,&Point);
	AddCord(&Point,&Point,pCenter);
	DrawVector(Point.X,Point.Y,OldPoint.X,OldPoint.Y,1);
	MoveCord(&OldPoint,&Point);
    }
}

CORDS Center;

void MakeClockFace()
{
    int i,Angle;
    CORDS Point,OldPoint;
    CORDS CA;
    char cBuffA[20];
    Center.X = 64;
    Center.Y = 64;
    CA.X = Center.X - 4;
    CA.Y = Center.Y - 5;
    if(DownLoadFont("csyslogo.bin",2))	/* Down load CSys labs logo */
    {
	exit(0);
    }
    SetEL(1);
    SetFontGrayPalet(0x4);
    SetFontPlane(1);
    SelectFont(2);
    SetXY(0,1);
    SetHeight(21);
    cBuffA[1] = 0;
    for(i=0;i<179;i++)
    {
	cBuffA[0] = 'A' + i%8;
	InputString(cBuffA);
    }
    SetHeight(0);
    cBuffA[0] = 'A' + i%8;
    InputString(cBuffA);

    SetFontGrayPalet(0xc);
    SelectFont(0);
    SetPlane(1);
    DrawBox(122,0,239,35,4);
    SetXY(126,5);
    InputString("  CSys Labs");
    SetXY(126,20);
    InputString("saves YOU time");

    SelectFont(0);
    SetFontGrayPalet(0xc);
    SetPlane(1);
    SetFontAttribute(1);

    // Place number around face
    for(i = 1,Angle = 30;Angle < 361;Angle+=30,i++)
    {
	Pol2Rec(Angle,55,&Point);
	AddCord(&Point,&Point,&CA);
	SetXY(Point.X,Point.Y);
	sprintf(cBuffA,"%d",i);
	InputString(cBuffA);
    }
    // Do center circle
    DrawCircle(&Center,3,30);
    // Do outer circle
    DrawCircle(&Center,63,10);

    // Do buttons
    LoadButtonBuffer("Set Hours");
    PlaceButton(0,6,3);
    LoadButtonBuffer(" Set Mins ");
    PlaceButton(0,6,4);
    SelectFont(1);
    LoadButtonBuffer("Prev");
    PlaceButton(0,6,5);
    LoadButtonBuffer(" End ");
    PlaceButton(0,8,5);

    DrawBlock(146,63,239,127,0);

    SetPlane(0);
}

void Hours(int HourAngle,bool bType)
{
    CORDS Point,OldPoint;
    Pol2Rec(HourAngle - 4,33,&Point);
    AddCord(&Point,&Point,&Center);
    MoveCord(&OldPoint,&Point);
    DrawVector(Point.X,Point.Y,Center.X,Center.Y,bType);

    Pol2Rec(HourAngle,40,&Point);
    AddCord(&Point,&Point,&Center);
    DrawVector(Point.X,Point.Y,OldPoint.X,OldPoint.Y,bType);
    MoveCord(&OldPoint,&Point);

    Pol2Rec(HourAngle + 4,33,&Point);
    AddCord(&Point,&Point,&Center);
    DrawVector(Point.X,Point.Y,OldPoint.X,OldPoint.Y,bType);
    DrawVector(Point.X,Point.Y,Center.X,Center.Y,bType);
}

void Minutes(int MinuteAngle,bool bType)
{
    CORDS Point,OldPoint;
    Pol2Rec(MinuteAngle - 3,40,&Point);
    AddCord(&Point,&Point,&Center);
    MoveCord(&OldPoint,&Point);
    DrawVector(Point.X,Point.Y,Center.X,Center.Y,bType);

    Pol2Rec(MinuteAngle,50,&Point);
    AddCord(&Point,&Point,&Center);
    DrawVector(Point.X,Point.Y,OldPoint.X,OldPoint.Y,bType);
    MoveCord(&OldPoint,&Point);

    Pol2Rec(MinuteAngle + 3,40,&Point);
    AddCord(&Point,&Point,&Center);
    DrawVector(Point.X,Point.Y,OldPoint.X,OldPoint.Y,bType);
    DrawVector(Point.X,Point.Y,Center.X,Center.Y,bType);
}

void Seconds(int SecondsAngle,bool bType)
{
    CORDS Point,OldPoint;
    Pol2Rec(SecondsAngle,55,&Point);
    AddCord(&Point,&Point,&Center);
    DrawVector(Point.X,Point.Y,Center.X,Center.Y,bType);
}

void srClock()
{
    long Time;
    struct tm * tm_now;
    int Angle,HandAngle;
    int i;
    CORDS CA;
    CORDS Point,OldPoint;
    static int HoursAngle;
    static int OldHoursAngle = -1;
    static int MinutesAngle;
    static int OldMinutesAngle = -1;
    static int SecondsAngle;
    static int OldSecondsAngle = -1;
    static bool bDisplaySec = 1;
    static bool bDisplayMin = 1;
    static bool bDisplayHour = 1;
    static bool bRemoveSec = 0;
    static bool bRemoveMin = 0;
    static bool bRemoveHour = 0;
    time_t t;

    t = time(NULL);
    tm_now = localtime(&t);

    // Do hours hand
    HoursAngle = (tm_now->tm_hour%12 * 30) + (tm_now->tm_min / 2);
    if((HoursAngle != OldHoursAngle) && (OldHoursAngle != -1))
    {
	bRemoveHour = 1;
    }

    // Do minutes hand
    MinutesAngle = (tm_now->tm_min * 6) + (tm_now->tm_sec / 12);
    if((MinutesAngle != OldMinutesAngle) && (OldMinutesAngle != -1))
    {
	bRemoveMin = 1;
    }

    // Do seconds hand
    SecondsAngle = (tm_now->tm_sec * 6);
    if((SecondsAngle != OldSecondsAngle) && (OldSecondsAngle != -1))
    {
	bRemoveSec = 1;
    }

    if(bRemoveSec)
    {
	bRemoveSec = 0;
	bDisplaySec = 1;
	Seconds(OldSecondsAngle,2);
    }
    if(bRemoveMin)
    {
	bRemoveMin = 0;
	bDisplayMin = 1;
	Minutes(OldMinutesAngle,2);
    }
    if(bRemoveHour)
    {
	bRemoveHour = 0;
	bDisplayHour = 1;
	Hours(OldHoursAngle,2);
    }
    if(bDisplayHour)
    {
	bDisplayHour = 0;
	Hours(HoursAngle,2);
	OldHoursAngle = HoursAngle;
    }
    if(bDisplayMin)
    {
	bDisplayMin = 0;
	Minutes(MinutesAngle,2);
	OldMinutesAngle = MinutesAngle;
    }
    if(bDisplaySec)
    {
	bDisplaySec = 0;
	Seconds(SecondsAngle,2);
	OldSecondsAngle = SecondsAngle;
    }
}