#ifndef  _SSERVPP_H
#define  _SSERVPP_H

/************************************************************************/
/*									*/
/*		Copyright 1991, CSys Labs Incorporated,			*/	
/*		   	238 So. Hillview Dr.				*/
/*		   	Milpitas, Ca. 95035				*/
/*									*/
/************************************************************************/

#include <csysdefs.h>

// SSERVPP.H is the header file for the sservpp.cpp
// Use this one to support CPP type serial interface

typedef struct sdefBase BASE;
struct sdefBase
{
    int Base;			/* HEX base address */
    int Vector;                 /* Default IRQ */
};

typedef struct sdefVector VECTOR;
struct sdefVector
{
    int Vector;			/* Vector table number */
    int IRQMask;                /* Mask for 8259 */
    STRING * pName;
};

typedef struct sdefBaud BAUD;
struct sdefBaud
{
    int Baud;
    STRING * pName;
};

/* Use the following definitions to open/close the serial port */

#define COM1 		0
#define COM2 		1
#define COM3		2
#define COM4		3

#define IRQ3		0
#define IRQ4		1
#define IRQDEF		-1

#define B300		0
#define B600		1
#define B1200		2
#define B2400		3
#define B4900		4
#define B9600		5
#define B19200		6
#define B38400		7

#define SIZEOFRB	0x1000		/* RXBuffer size */
#define RBMASK		(SIZEOFRB - 1)  /* RXBuffer index mask */

/* Special code definitions */
#define DC1 0x11	/* XON */
#define DC2 0x12        /* Key press signal */
#define DC3 0x13        /* XOFF signal */
#define XON DC1
#define XOFF DC3
#define RTSCTS 1
#define XONXOFF 2

/* Static vars used by routines */
extern uchar RBuffer[];		/* Recieve buffer */
extern uint RBIn;		/* Input pointer */
extern bool bXON;		/* Set to use XON protocall */
extern int Save_Ocw1;
extern int UartBase;
extern int VectNumb;
extern uchar * pTX;
extern uint BlockCount;
extern bool bProtocol;		/* Default to no protocol */
extern bool bIER;
extern bool bKPS;		/* Key press signal */
extern bool bTXBusy;	 	// 1 means TX is busy sending, do not touch

class tvms;			// Define tvms for constructor use.

void interrupt far sserverint(...);

class sserver
{
  protected:

    /* vars used by routines */
    tvms * pTvms;		// Attached TVMS stream

  public:
    sserver(tvms * pT);			// Default constructor
    bool OpenServer(int Port,int IRQ,uint iBaud,STRING * Protocol);
    static void CloseServer();
    static void SendString(STRING * pS);/* Puts a string to the UART */
    static void SendBlock(uchar * pC,uint Count);/* Sends a block of characters to the UART */
};

#endif  /* _SSERVPP_H */
