#ifndef  _TVIOPP_H
#define _TVIOPP_H

/************************************************************************/
/*									*/
/*		Copyright 1991, CSys Labs Incorporated,			*/
/*		   	238 So. Hillview Dr.				*/
/*		   	Milpitas, Ca. 95035				*/
/*									*/
/************************************************************************/

#include <stdio.h>
#include <csysdefs.h>
#include <sservpp.h>

/* tviopp.h is the tviopp.cpp header file.  It allows multiple TVM modules
to exist in a system */

// TVM is the virtual definitions for the tvm routines.  These
// routine are common between TVMP and TVMS

/*********************** TVM **************************/

// Class tvm suppors a put virtual function for both serial and paralell
// versions.

typedef class tvm TVM;
class tvm
{
  public:
    enum {TVM2464,TVM24128} eModuleType;
    int MaxX,MaxY;			// Max x and max y
    void SetModuleType();               // This determins the module type
    virtual bool CheckKEYPRESS(void) = 0;
    virtual bool CheckCPUBUSY(){return 0;};
    virtual bool TestError(STRING * pString,...) = 0;
    virtual bool SelectFont(uchar xxx) = 0;
    virtual void SetFontAttribute(uchar ab) = 0;
    virtual void SetFontGrayPalet(uchar Palet) = 0;
    virtual void SetFontPlane(uchar Plane) = 0;
    virtual bool DownLoadFont(STRING * sFileName,uint FontNumber) = 0;
    virtual bool GetFontChar(FILE * pFile,uchar * pChar) = 0;
    virtual void SetXY(uchar XPos,uchar YPos) = 0;
    virtual bool ReadXY(uchar * XPos,uchar * YPos) = 0;
    virtual void CursorUp(void) = 0;
    virtual void CursorDown(void) = 0;
    virtual void CursorLeft(void) = 0;
    virtual void CursorRight(void) = 0;
    virtual void SetX(uchar XPos) = 0;
    virtual void SetY(uchar YPos) = 0;
    virtual void CursorAttribute(uchar abc) = 0;
    virtual void SetTextWindow(uchar AX,uchar AY,uchar BX,uchar BY) = 0;
    virtual void SetPitch(uchar Pitch) = 0;
    virtual void SetHeight(uchar Height) = 0;
    virtual void InputString(STRING * String) = 0;
    virtual void DrawBox(uchar AX,uchar AY,uchar BX,uchar BY,uchar TTTF) = 0;
    virtual void DrawBlock(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT) = 0;
    virtual void DrawHoriz(uchar XPos,uchar YPos,char Length,uchar TT) = 0;
    virtual void DrawVert(uchar XPos,uchar YPos,char Length,uchar TT) = 0;
    virtual void DrawVector(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT) = 0;
    virtual void SetPixel(uchar XPos,uchar YPos,uchar TT) = 0;
    virtual void MoveBlockVert(uchar AX,uchar AY,uchar BX,uchar BY,char Distance) = 0;
    virtual void MoveBlockHoriz(uchar AX,uchar AY,uchar BX,uchar BY,char Distance) = 0;
    virtual bool PlaceButton(uchar Code,uchar Col,uchar Row) = 0;
    virtual void LoadButtonBuffer(STRING * String) = 0;
    virtual bool GetButtonSize(uchar KeyCode,uchar * RData) = 0;
    virtual bool PlacePhantomButton(uchar KeyCode,uchar Col,uchar Row,uchar Length) = 0;
    virtual bool DeleteButton(uchar KeyCode) = 0;
    virtual void DeleteAllButtons(void) = 0;
    virtual uchar ReadKeyCode(void) = 0;
    virtual void SetButtonAttribute(uchar abc) = 0;
    virtual void SetButtonGrayPalet(uchar Palet) = 0;
    virtual void SetButtonPlane(uchar Plane) = 0;
    virtual void BlankDisplay(void) = 0;
    virtual void ClearDisplay(void) = 0;
    virtual void Refresh(void) = 0;
    virtual void SetAutoRefresh(uchar ab) = 0;
    virtual bool DumpDisplayRAM(STRING * sFileName) = 0;
    virtual bool LoadDisplayRAM(STRING * sFileName) = 0;
    virtual void LoadGrayDither(uchar GD1,uchar GD2) = 0;
    virtual void ReverseVideo(uchar Reverse) = 0;
    virtual void SetGrayPalet(uchar Palet) = 0;
    virtual void SetPlane(uchar Plane) = 0;
    virtual void SoftReset(void) = 0;
    virtual void SetContrast(uchar) = 0;
    virtual void SetEL(uchar a) = 0;
    virtual void NOP(void) = 0;
    virtual void SetBeeper(uchar a) = 0;
    virtual bool ReadKeyMatrix(uchar * RData) = 0;
    virtual bool ReadAuxPort(uchar * pAuxData) = 0;
    virtual void WriteAuxPort(uchar AuxData) = 0;

};

/*********************** TVMP **************************/

// TVMP supports the parallel interface version

typedef class tvmp TVMP;
class tvmp : public tvm
{
  protected:
    int Base;				// Module base address
  public:
    tvmp()
    {
	Base = 0x300;
	eModuleType = TVM2464;
    };
    bool StartUp(int iBase = 0);
    bool Echo();
    void SetBase(uint B);
    uchar GetStatus(void);
    bool CheckIBF(void);
    bool CheckOBF(void);
    bool CheckERROR(void);
    bool CheckCPUBUSY(void);
    bool SendChar(uchar uAddr,uchar cChar);
    bool GetData(uchar * pData);

    bool CheckKEYPRESS(void);
    bool TestError(STRING * pString,...);
    bool SelectFont(uchar xxx);
    void SetFontAttribute(uchar ab);
    void SetFontGrayPalet(uchar Palet);    void SetFontPlane(uchar Plane);
    bool DownLoadFont(STRING * sFileName,uint FontNumber);
    bool GetFontChar(FILE * pFile,uchar * pChar);
    void SetXY(uchar XPos,uchar YPos);
    bool ReadXY(uchar * XPos,uchar * YPos);
    void CursorUp(void);
    void CursorDown(void);
    void CursorLeft(void);
    void CursorRight(void);
    void SetX(uchar XPos);
    void SetY(uchar YPos);
    void CursorAttribute(uchar abc);
    void SetTextWindow(uchar AX,uchar AY,uchar BX,uchar BY);
    void SetPitch(uchar Pitch);
    void SetHeight(uchar Height);
    void InputString(STRING * String);
    void DrawBox(uchar AX,uchar AY,uchar BX,uchar BY,uchar TTTF);
    void DrawBlock(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT);
    void DrawHoriz(uchar XPos,uchar YPos,char Length,uchar TT);
    void DrawVert(uchar XPos,uchar YPos,char Length,uchar TT);
    void DrawVector(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT);
    void SetPixel(uchar XPos,uchar YPos,uchar TT);
    void MoveBlockVert(uchar AX,uchar AY,uchar BX,uchar BY,char Distance);
    void MoveBlockHoriz(uchar AX,uchar AY,uchar BX,uchar BY,char Distance);
    bool PlaceButton(uchar Code,uchar Col,uchar Row);
    void LoadButtonBuffer(STRING * String);
    bool GetButtonSize(uchar KeyCode,uchar * RData);
    bool PlacePhantomButton(uchar KeyCode,uchar Col,uchar Row,uchar Length);
    bool DeleteButton(uchar KeyCode);
    void DeleteAllButtons(void);
    uchar ReadKeyCode(void);
    void SetButtonAttribute(uchar abc);
    void SetButtonGrayPalet(uchar Palet);
    void SetButtonPlane(uchar Plane);
    void BlankDisplay(void);
    void ClearDisplay(void);
    void Refresh(void);
    void SetAutoRefresh(uchar ab);
    bool DumpDisplayRAM(STRING * sFileName);
    bool LoadDisplayRAM(STRING * sFileName);
    void LoadGrayDither(uchar GD1,uchar GD2);
    void ReverseVideo(uchar Reverse);
    void SetGrayPalet(uchar Palet);
    void SetPlane(uchar Plane);
    void SoftReset();
    void SetContrast(uchar);
    void SetEL(uchar a);
    void NOP();
    void SetBeeper(uchar a);
    bool ReadKeyMatrix(uchar * RData);
    bool ReadAuxPort(uchar * pAuxData);
    void WriteAuxPort(uchar AuxData);
};

/*********************** TVMS **************************/

#define NUMERIC		0		/* Numeric token */
#define	ERROR		1               /* Error token */
#define	ECHO		2               /* Echo token */
#define	KEYPRESSED	3               /* Key pressed token */
#define KEYCODE		4		/* Key code */

#define SIC		10		/* Serial interface command */
#define NTOK		-1              /* NULL Token */

// TVMS supports the serial interface version

typedef class tvms TVMS;
class tvms : public tvm, public sserver
{
  private:
    STRING text[80];		/* lexeme buffer */
    STRING text2[80];		/* Echo lexeme buffer */

    /* vars used in yylex */
    int yystate;		/* accept state */
    uint sMark;			/* Lexeme Start index mark */
    uint Next;			/* Net buffer index mark */
  public:
    bool Verbose;		// Verbose for routines
    tvms();			// Constructor
    ~tvms();
    void ssprintf(STRING * sBuff,STRING * pFormat,...);
    int yylex();
    uint Echo();
    void SetLowThreshold(uint Threshold);
    void SetHighThreshold(uint Threshold);
    void EnableXON();
    void ResetXON();
    bool GetData(uchar * pData);
    bool StartUp(int Port,int IRQ,uint iBaud,STRING * Protocol);

    bool CheckKEYPRESS(void);
    bool TestError(STRING * pString,...);
    bool SelectFont(uchar xxx);
    void SetFontAttribute(uchar ab);
    void SetFontGrayPalet(uchar Palet);
    void SetFontPlane(uchar Plane);
    bool DownLoadFont(STRING * sFileName,uint FontNumber);
    bool GetFontChar(FILE * pFile,uchar * pChar);
    void SetXY(uchar XPos,uchar YPos);
    bool ReadXY(uchar * XPos,uchar * YPos);
    void CursorUp(void);
    void CursorDown(void);
    void CursorLeft(void);
    void CursorRight(void);
    void SetX(uchar XPos);
    void SetY(uchar YPos);
    void CursorAttribute(uchar abc);
    void SetTextWindow(uchar AX,uchar AY,uchar BX,uchar BY);
    void SetPitch(uchar Pitch);
    void SetHeight(uchar Height);
    void InputString(STRING * String);
    void DrawBox(uchar AX,uchar AY,uchar BX,uchar BY,uchar TTTF);
    void DrawBlock(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT);
    void DrawHoriz(uchar XPos,uchar YPos,char Length,uchar TT);
    void DrawVert(uchar XPos,uchar YPos,char Length,uchar TT);
    void DrawVector(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT);
    void SetPixel(uchar XPos,uchar YPos,uchar TT);
    void SetGrayPalet(uchar Palet);
    void SetPlane(uchar Plane);
    bool PlaceButton(uchar Code,uchar Col,uchar Row);
    void LoadButtonBuffer(STRING * String);
    bool GetButtonSize(uchar KeyCode,uchar * RData) ;
    bool PlacePhantomButton(uchar KeyCode,uchar Col,uchar Row,uchar Length);
    bool DeleteButton(uchar KeyCode);
    void DeleteAllButtons(void);
    uchar ReadKeyCode(void);
    void SetButtonAttribute(uchar abc);
    void SetButtonGrayPalet(uchar Palet);
    void SetButtonPlane(uchar Plane);
    void BlankDisplay(void);
    void ClearDisplay(void);
    void Refresh(void);
    void SetAutoRefresh(uchar ab);
    bool DumpDisplayRAM(STRING * sFileName);
    bool LoadDisplayRAM(STRING * sFileName);
    void MoveBlockVert(uchar AX,uchar AY,uchar BX,uchar BY,char Distance);
    void MoveBlockHoriz(uchar AX,uchar AY,uchar BX,uchar BY,char Distance);
    void LoadGrayDither(uchar GD1,uchar GD2);
    void ReverseVideo(uchar bReverse);
    void SoftReset(void);
    void SetContrast(uchar Data);
    void SetEL(uchar a);
    void NOP(void);
    void SetBeeper(uchar a);
    bool ReadKeyMatrix(uchar * RData);
    bool ReadAuxPort(uchar * pAuxData);
    void WriteAuxPort(uchar AuxData);
};

#endif  /* _TVIOPP_H */
