/************************************************************************/
/*									*/
/*		Copyright 1991, CSys Labs Incorporated,			*/	
/*		   	238 So. Hillview Dr.				*/
/*		   	Milpitas, Ca. 95035				*/
/*									*/
/************************************************************************/

/* SSERVER.C is a interupt driven server for the COM ports. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <bios.h>
#include <dos.h>
#include <csysdefs.h>
#include <sserver.h>

#define UART_RBR	(UartBase + 0)
#define UART_THR	(UartBase + 0)
#define UART_DLL	(UartBase + 0)
#define UART_IER	(UartBase + 1)
#define UART_DLM	(UartBase + 1)
#define UART_IIR	(UartBase + 2)
#define UART_LCR	(UartBase + 3)
#define UART_MCR	(UartBase + 4)
#define UART_LSR	(UartBase + 5)
#define UART_MSR	(UartBase + 6)
#define D8BIT		0x03
#define D1STOP		0x00
#define NOPARITY	0x00
#define TXHRE		(Status &0x4000)/* Transmit holding register is empty */
#define CTS		(Status & 0x10)	/* Clear to send line */
#define DATA_READY	(Status & 0x100)/* Tests if RX data is ready */
#define OCW1		0x21
#define OCW2		0x20
#define OCW3		0x20
#define ENIRQ4()	outportb(OCW1,0xef & inportb(OCW1))
#define GETIRMASK()	inportb(OCW1)
#define EOI()		outportb(OCW3,0x20)
#define SETTXINT()	outportb(UART_IER,bIER|=0x2)	/* Enable interupt from UART */
#define RESETTXINT()	outportb(UART_IER,bIER&=0xfd)	/* Disable interupt from UART */
#define TESTTX		(inportb(UART_IIR)==0x2)/* Test for TX Hoding empty */
#define TESTRX		(inportb(UART_IIR)==0x4)/* Test for RX Full */
#define TESTRLS		(inportb(UART_IIR)==0x0)/* Test for RLS */
#define TESTMS		(inportb(UART_IIR)==0x6)/* Test for MS */
#define TESTIRP		(!(inportb(UART_IIR)&0x1))/* Test for interupt pending */
#define SETRTS()	outportb(UART_MCR,0xa);		/* Set RTS */
#define RESETRTS()	outportb(UART_MCR,0x8);		/* Reset RTS */

typedef struct sdefBase BASE;
struct sdefBase
{
    int Base;			/* HEX base address */
    int Vector;                 /* Default IRQ */
};

typedef struct sdefVector VECTOR;
struct sdefVector
{
    int Vector;			/* Vector table number */
    int IRQMask;                /* Mask for 8259 */
    STRING * pName;
};

typedef struct sdefBaud BAUD;
struct sdefBaud
{
    int Baud;
    STRING * pName;
};

BASE Base[] =
{
    {0x3f8,IRQ4},
    {0x2f8,IRQ3},
    {0x3e8,IRQ4},
    {0x2f8,IRQ3}
};

VECTOR Vector[] =
{
    {0xb,0xf7,"IRQ3"},		       /* IRQ3 */
    {0xc,0xef,"IRQ4"}                  /* IRQ4 */
};

BAUD Baud[] =
{
    {384,"300"},
    {192,"192"},
    {96,"1200"},
    {48,"2400"},
    {24,"4800"},
    {12,"9600"},
    {6,"19200"},
    {3,"38400"}
};

/* Use these global variables to get to the buffer */
uchar RBuffer[SIZEOFRB];	/* Recieve buffer */
uint RBIn = 0;			/* Input pointer */
bool bTXBusy = 0;		/* 1 means TX is busy sending, do not touch
				buffer area untill bTXBusy = 0 */
bool bKPS = 0;			/* Key press signal */

/* static vars used by routines */
static int Save_Ocw1;
static int UartBase;
static int VectNumb;
STRING * pTX;
static uint BlockCount = 0;
static bool bProtocol = 0;	/* Default to no protocol */
static bool bIER = 0;
bool bXON = 1;

void interrupt sserver()
{
    int IIRSave;
    uchar RXChar;
    while(!((IIRSave = inportb(UART_IIR))&0x1))/* Loop till all interupts are served */
    {
	if(IIRSave == 0x2)	/* See if TX interupt */
	{
	    if(!bXON)		/* XOFF now so disable sending */
	    {
		RESETTXINT();
	    }
	    else if(!BlockCount && !*pTX)
	    {
		RESETTXINT();
		RESETRTS();
	    }
	    else
	    {
		if(!BlockCount)   	/* See if not sending a block */
		{
		    if(*pTX)		/* See if data not NULL */
		    {
			outportb(UART_THR,*pTX++);
		    }
		    if(!*pTX)
		    {
			bTXBusy = 0;	/* TX Not busy now */
		    }
		}
		else			/* Sending block data */
		{
		    outportb(UART_THR,*pTX++);/* Send and inc pointer */
		    BlockCount--;
		    if(!BlockCount)
		    {
			bTXBusy = 0;	/* TX Not busy now */
		    }
		}
	    }
	}
	else if(IIRSave == 0x4)		/* RX Buffer full */
	{
	    RXChar = inportb(UART_RBR);	/* Get character */
	    if((bProtocol == XONXOFF)&&(RXChar == XOFF))
	    {
		bXON = 0;
		RESETTXINT();	/* Disable TX Interupt to stop TX */
	    }
	    else if((bProtocol == XONXOFF)&&(RXChar == XON))
	    {
		bXON = 1;
		SETTXINT();	/* Re-enable TX Interupt */
	    }
	    else if(RXChar == DC2)	/* See if KPS signal */
	    {
		bKPS = 1;
	    }
	    else
	    {
		RBuffer[RBIn++ & RBMASK] = RXChar;
	    }
	}
	else if(IIRSave == 0x6)	/* Reciever line status change */
	{
	    inportb(UART_LSR);
	}
	else if(IIRSave == 0x0)	/* Modem Status */
	{
	    if(bProtocol == RTSCTS)
	    {
		if(inportb(UART_MSR) & 0x10)	/* Test CTS */
		{
		    bXON = 1;
		    SETTXINT();
		}
		else
		{
		    bXON = 0;
		    RESETTXINT();
		}
	    }
	}
    }
    EOI();			/* Re-enable interupts */
}

/* OpenServer opens the serial port for operation and installes the
interupt server for the serial port */
bool OpenServer(int Port,int IRQ,uint iBaud,STRING * Protocol)
{
    if(Port > COM4)
    {
	printf("COM_ERROR: Illegal COM specified, COM1-4 allowed.\n");
	return 1;
    }
    else
    {
	UartBase = Base[Port].Base;
	printf("COM%d Base is %x\n",Port,UartBase);
    }
    if(IRQ >= 0)
    {
	if(IRQ > 1)
	{
	    printf("COM_ERROR: Illegal IRQ specified, Only IRQ3 or IRQ4.\n");
	    return 1;
	}
	else
	{
	    VectNumb = IRQ;
	}
    }
    else				/* Load default vector */
    {
	VectNumb = Base[Port].Vector;
    }
    if(iBaud > 7)
    {
	printf("COM_ERROR: Illegal BAUD specified, 0 - 1 allowed.\n");
	return 1;

    }
    printf("%s is vector %x\n",Vector[VectNumb].pName,Vector[VectNumb].Vector);
    printf("%s Baud is selected.\n",Baud[iBaud].pName);
    setvect(Vector[VectNumb].Vector,sserver);/* Install sserver */
    outportb(UART_LCR,0x80|D8BIT|D1STOP|NOPARITY);/* Enable load baud, set to 8 bits */
    outportb(UART_DLL,(uchar)Baud[iBaud].Baud);	/* Send baud LSB */
    outportb(UART_DLM,(uchar)(Baud[iBaud].Baud>>8));  /* Send baud MSB */
    outportb(UART_LCR,0x00|D8BIT|D1STOP|NOPARITY);/* Disable load baud, set to 8 bits, no parity, 1 stop */
    inportb(UART_RBR);		/* Read data to clear read register */
    outportb(UART_IER,bIER=0x1);	/* Enable recieve interupt */
    outportb(UART_MCR,0x8);	/* Enable interupt from UART */
    Save_Ocw1 = inportb(OCW1);	/* Get current mask status */
    outportb(OCW1,Vector[VectNumb].IRQMask & inportb(OCW1));/* Enable 8259 mask */
    SendString("\nSR\n");	/* Terminate any old command, Reset module */
    delay(500);			/* Give time for TVM module to reset */
    RBIn = 0;			/* Reset RBIn to clear RBuffer */
    if(Protocol)		/* See if Protocol specified */
    {
	if(!strcmp(Protocol,"RTSCTS"))
	{
	    SendString("@R\n");	/* Disable XON */
	    while(bTXBusy);	/* Wait till data sent befor enabling RTS/CTS */
	    bProtocol = RTSCTS;	/* RTS/CTS Hardware handshaking */
	    outportb(UART_IER,bIER=0x9);/* Enable MSR interupt */
	    printf("RTS/CTS Hardware Hanshaking\n");
	}
	else if(!strcmp(Protocol,"XONXOFF"))
	{
	    SendString("@X\n");	/* Enable XON */
	    bProtocol = XONXOFF;/* XONXOFF software handshaking */
	    printf("XON/XOFF Software Handshaking\n");
	}
	else
	{
	    printf("No Handshaking\n");
	}
    }
    return(0);
}
void CloseServer()
{
    outportb(OCW1,Save_Ocw1);	/* Restore status of OCW1 */
    outportb(UART_IER,bIER=0x0);/* Disable UART interupts */
    outportb(UART_MCR,0x0);	/* Disable interupt from UART */
}
void SendString(STRING * pS)
{
    while(bTXBusy);		/* Wait till not busy */
    if(bProtocol == RTSCTS)SETRTS();/* Set RTS signal */
    pTX = pS;			/* Set pointer flag */
    bTXBusy = 1;		/* Set busy flag */
    SETTXINT();			/* Enable UART to transmit */
}
void SendBlock(char * pC,uint Count)
{
    while(bTXBusy);		/* Wait till not busy */
    if(bProtocol == RTSCTS)SETRTS();/* Set RTS Signal */
    pTX = pC;			/* Set pointer flag */
    bTXBusy = 1;		/* Set busy flag */
    BlockCount = Count;		/* Set block count */
    SETTXINT();			/* Enable UART to transmit */
}
