/************************************************************************/
/*									*/
/*		Copyright 1991, CSys Labs Incorporated,			*/	
/*		   	238 So. Hillview Dr.				*/
/*		   	Milpitas, Ca. 95035				*/
/*									*/
/************************************************************************/

/* TVIO has all the Touch Vision I/O routines */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <dos.h>
#include <stdarg.h>
#include <csysdefs.h>
#include <tvio.h>
#include <bios.h>
#include <dos.h>

/* Modified for TVM24128 2/14/91 */

uint Base;			/* Base IO port address */
int ModuleType = TVM2464_TYPE;

void Delay(unsigned int time_in_milliseconds)
{
    long Start_Time;
    long Stop_Time;
    Start_Time = biostime(0,0);
    /* Divide time_in_milliseconds by 54.9 ms per tick */
    Stop_Time = Start_Time + (time_in_milliseconds / 55);
    while(Stop_Time >= biostime(0,0));
    return;
}

/* StartUp reads a file "base.dat" to see if a BASE address is specified.
If so, it uses this as it's initial assumption of the base address to
try to establish communication with the TVM.  If this file does not exist
or it is in error it first looks at address 0x300 and the 0x307 to try to
establish communication.  After this, it writes the file base.dat so it has
the BASE address next time. StartUp returns 0 if it finds the TVM.  Else
if return 1. */
bool StartUp(void)
{
    FILE * fpBase;			/* File pointer to base */
    printf("Opening file base.dat...");
    if((fpBase = fopen("base.dat","rt")) != NULL)
    {
	printf("Reading...");
	if(fscanf(fpBase,"%x",&Base))	/* Get base address */
	{
	    fclose(fpBase);		/* Close file */
	    if(Echo())
	    {
		printf("Complete, BASE = %x\n",Base);
	        return(0);
	    }
	    else
	    {
		printf("\nERROR:Could not establish contact at BASE=%x\n",Base);
	    }
	}
	else
	{
	    fclose(fpBase);		/* Close file */
	    printf("ERROR: Reading file base.dat\n");
	}
    }
    /* POLL TVM board for valid BASE address */
    else
    {
        printf("File not found\n");
    }
    printf("Will POLL for TVM interface...");
    for(Base = 0x300;Base < 0x319;Base +=8)
    {
	printf("%x...",Base);
	if(Echo())
	{
	    printf("Found BASE at %x\n",Base);
	    printf("Creating file base.dat...");
	    if((fpBase = fopen("base.dat","wt")) == NULL)
	    {
		printf("ERROR: Could not create file\n");
		return(0);
	    }
	    else
	    {
		fprintf(fpBase,"%x",Base);
	        fclose(fpBase);		/* Close file */
		printf("Complete\n");
		return(0);
	    }
	}
    }
    printf("\nCould not establish contact.\n");
    return(1);
}
/* Echo sends a SetXY command to the TVM module.  It then trys to read the
command.  If if does not read the given XY cords, it returns a zero, else it
returns a 1 */
bool Echo()
{
    uchar i;
    uchar x,y;
    SoftReset();
    Delay(1000);			/* Wait 1 second */
    if(CheckOBF())			/* See if output buffer is full */
    {
	printf("\nOBF\ Valid, Clearing flag\n");
	GetData(&i);
    }
    SetXY(100,50);			/* Test to see if TVM is connected */
    ReadXY(&x,&y);
    if((x != 100)||(y != 50))
    {
	return(0);
    }
    SetXY(50,25);			/* Test to see if TVM is connected */
    ReadXY(&x,&y);
    if((x != 50)||(y != 25))
    {
	return(0);
    }
    CheckModuleType();			/* This sets the ModuleType global variable. */
    return(1);
}
void CheckModuleType()	/* This sets the ModuleType global variable. */
{
    uchar x,y;
    SetXY(100,50);
    ReadXY(&x,&y);
    if((x == 100)||(y == 50))
    {
	ModuleType = TVM2464_TYPE;
    }
    SetXY(150,100);
    ReadXY(&x,&y);
    if((x == 100)||(y == 100))
    {
	ModuleType = TVM24128_TYPE;
    }
}
/* SetBase sets the Base address for the IO card */
void SetBase(uint B)
{
    Base = B;
}
/* GetStatus returns the status register */
uchar GetStatus(void)
{
    return(inp(3+Base));
}

/* CheckIBF tests to see if the input buffer is full.  It return 1 if IBF is
set.  It return 0 if IBF is empty. */
bool CheckIBF(void)
{
    if(GetStatus() & 0x01)	/* return IBF flag */
    {
	return(1);
    }
    return(0);
}

/* CheckOBF tests to see if the output buffer is full.  It return 1 if OBF is 
full.  It return 0 if OBF is empty. */
bool CheckOBF(void)
{
    if(GetStatus() & 0x02)
    {
	return(0);
    }
    return(1);
}

/* CheckERROR tests to see if the Error flag is set.  It return 1 if ERROR is
set.  It return 0 if ERROR is not set. */
bool CheckERROR(void)
{
    if(GetStatus() & 0x04)	/* return ERROR flag */
    {
	return(1);
    }
    return(0);
}

/* CheckKEYPRESS tests to see if the KEYPRESS flag is set.  It return 1 if
set.  It return 0 if not set. */
bool CheckKEYPRESS(void)
{
    if(GetStatus() & 0x08)	/* return KEYPRESS flag */
    {
	return(1);
    }
    return(0);
}

/* CheckCPUBUSY tests to see if the CPUBUSY flag is set.  It return 1 if 
set.  It return 0 if not set. */
bool CheckCPUBUSY(void)
{
    if(GetStatus() & 0x10)	/* return CPUBUSY flag */
    {
	return(1);
    }
    return(0);
}

/* SendChar send a character to the TV Module.  It return 0 it sending was
successful.  It returns 1 if error (stayed busy to long.  It gives about
.5 sec to send data. */
bool SendChar(uchar uAddr,uchar cChar)
{
    uint i = 0;
    while(CheckIBF())
    {
	if(++i > 50000)
	{
	    printf("ERROR: SendChar timeout\n");
	    return(1);
        }
    }
    outp(uAddr+Base,cChar); 		/* Send data */
    return(0);
}
/* GetData gets char data from TPDI PPI. */
/* return 0 for successful transfer with valid data in *pData. */
/* If timeout error (data never ready), it returns -1 and data returned is -1.
If ERROR flag set but data was ready it returns 1 with valid data */
bool GetData(uchar * pData)
{
    uint i=0;
    while(!CheckOBF())
    {
	if(++i > 50000)			/* Loop timeout, return -1 */
	{
	    printf("ERROR: GetData timeout\n");
	    *pData = 0xff;
	    return(-1);
	}
    }
    *pData = inp(Base);		/* Get data */
    return(CheckERROR());		/* Return ERROR flag */
}
/* Test error waits till the CPU is not busy.  It then checks the ERROR flag.
If an ERROR has occured, it sends the message to the PC and returns 1.  If no
error it returns 0 */
bool TestError(STRING * pString,...)
{
    char Buffer[80];
    va_list argptr;
    while(CheckCPUBUSY());		/* Wait till not busy */
    if(CheckERROR())
    {
	if(pString)
	{
    	    va_start(argptr,pString);
	    vsprintf(Buffer,pString,argptr);
	    printf("ERROR: %s\n",Buffer);
	    va_end(argptr);
	}
	return(1);
    }
    return(0);
}

/********** Font Selection **********/
bool SelectFont(uchar xxx)
{
    SendChar(0,xxx & 0x7);	/* Send code */
    SendChar(1,0xff);	 	/* Send bogus code to make sure font error
				flag can set if required.  This code will
				be ignored and is used only to delay the
				software untill the SetFont instruction can
				set the flags. */
    return(TestError("NULL Font"));
}
void SetFontAttribute(uchar ab)
{
    SendChar(0,0x18 + (ab & 0x3));
}
void SetFontGrayPalet(uchar Palet) 
{
    SendChar(0,0xb0 + (Palet & 0x0f));	/* Set Font Gray Palet */
}
void SetFontPlane(uchar Plane) 
{
    SendChar(0,0x1c + (Plane & 0x01));	/* Set Font Palet */
}
/* DownLoadFont loads a font from a file into font memory.  It returns a
0 for a successful load.  Returns a zero if an error. sFileName contains the
name of the file to open. */
bool DownLoadFont(STRING * sFileName,uint FontNumber)
{
    FILE * fpFile;
    uchar a;
    if((fpFile = fopen(sFileName,"rb")) == NULL)
    {
        printf("ERROR: File %s not found\n",sFileName);
        return(1);
    }
    SendChar(0,0x08 + (FontNumber & 0x07));	/* Send command */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
        return(1);
    }
    SendChar(1,a);		/* Send Size X */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
        return(1);
    }
    SendChar(1,a);		/* Send Size Y */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
        return(1);
    }
    SendChar(1,a);		/* Send Offset */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
        return(1);
    }
    SendChar(1,a);		/* Send Decenders */
    while(1)
    {
        a = fgetc(fpFile);
        if(feof(fpFile))
        {
	    if(TestError("Down load error"))
	    {
		fclose(fpFile);
		return(1);
	    }
	    fclose(fpFile);
	    return(0);
        }
        if(ferror(fpFile))
        {
       	    printf("ERROR: Error reading file %s\n",sFileName);
	    fclose(fpFile);
	    return(1);
        }
        SendChar(1,a);	/* Send FONT data to TPDI */
    }
}
bool GetFontChar(FILE * pFile,uchar * pChar)
{
    *pChar = fgetc(pFile);
    if(feof(pFile))
    {
    	return(1);
    }
    if(ferror(pFile))
    {
        printf("\nERROR: Error reading file\n");
        return(1);
    }
    return(0);
}

/******** Cursor Positioning ********/
void SetXY(uchar XPos,uchar YPos)
{
    SendChar(0,0x20);
    SendChar(1,XPos);
    SendChar(1,YPos);
}
bool ReadXY(uchar * XPos,uchar * YPos)
{
    SendChar(0,0x21);
    if(GetData(XPos) || GetData(YPos))
    {
	return(1);
    }
    return(0);
}
void CursorUp(void)
{
    SendChar(0,0x22);
}
void CursorDown(void)
{
    SendChar(0,0x23);
}
void CursorLeft(void)
{
    SendChar(0,0x24);
}
void CursorRight(void)
{
    SendChar(0,0x25);
}
void SetX(uchar XPos)
{
    SendChar(0,0x26);
    SendChar(1,XPos);
}
void SetY(uchar YPos)
{
    SendChar(0,0x27);
    SendChar(1,YPos);
}
void CursorAttribute(uchar abc)
{
    SendChar(0,0x10 + (abc & 0x7));
}

/********* Text Configuration *********/
void SetTextWindow(uchar AX,uchar AY,uchar BX,uchar BY)
{
    SendChar(0,0x28);	/* Send command  */
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
}
void SetPitch(uchar Pitch)
{
    SendChar(0,0x2a);	/* Send Pitch */
    SendChar(1,Pitch);
}
void SetHeight(uchar Height)
{
    SendChar(0,0x2b);	/* Send Height */
    SendChar(1,Height);
}

/********* Text Input *********/
/* InputString sends a string to the display */
void InputString(STRING * String)
{
    SendChar(0,0x2c);		/* Write string */
    while(*String)		/* Go till NULL terminator */
    {
        if(*String == '\n')
	{

	    SetX(0);
	    CursorDown();
    	    SendChar(0,0x2c);		/* Write string */
	}
	else
	{
            SendChar(1,*String);	/* Send all char's */
	}
	String++;		/* Incriment pointer */
    }
}

/********* Graphics Input **********/
void DrawBox(uchar AX,uchar AY,uchar BX,uchar BY,uchar TTTF)
{
    SendChar(0,0x40 + (TTTF & 0x0f));
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
}
void DrawBlock(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT)
{
    SendChar(0,0x60 + (TT & 0x03));
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
}
void DrawHoriz(uchar XPos,uchar YPos,char Length,uchar TT)
{
    SendChar(0,0x64+(TT&0x03));
    SendChar(1,XPos);
    SendChar(1,YPos);
    SendChar(1,Length);
}
void DrawVert(uchar XPos,uchar YPos,char Length,uchar TT)
{
    SendChar(0,0x68+(TT&0x03));
    SendChar(1,XPos);
    SendChar(1,YPos);
    SendChar(1,Length);
}
void DrawVector(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT)
{
    SendChar(0,0x6c + (TT & 0x03));
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
}
void SetPixel(uchar XPos,uchar YPos,uchar TT)
{
    SendChar(0,0x70+(TT & 0x03));
    SendChar(1,XPos);
    SendChar(1,YPos);
}
void SetGrayPalet(uchar Palet) 
{
    SendChar(0,0xc0 + (Palet & 0x0f));	/* Set Gray Palet */
}
void SetPlane(uchar Plane) 
{
    SendChar(0,0x78 + (Plane & 0x01));	/* Set Plane */
}

/******* Button Input *********/

/* PlaceButton return 0 if putton placement was successful.  Returns 1 if error
and button cound not be placed. */
bool PlaceButton(uchar Code,uchar Col,uchar Row)
{
    SendChar(0,0x30);		/* Put Button */
    SendChar(1,Code);		/* Send Key Code */
    SendChar(1,((Row<<4)&0x70)|(Col&0xf));	/* Send position */
    return(TestError(NULLP));
}
void LoadButtonBuffer(STRING * String)
{
    register STRING * pS = String;
    SendChar(0,0x31);		/* Send Button String command */
    while(*pS)
    {
        SendChar(1,*pS++);
    }
}
/* Return 0 if no error, 1 if error */
bool GetButtonSize(uchar KeyCode,uchar * RData)
{
    SendChar(0,0x32);			/* Send OP */
    SendChar(1,KeyCode);		/* Send Code */
    return(GetData(RData));		/* Get Data */
}
bool PlacePhantomButton(uchar KeyCode,uchar Col,uchar Row,uchar Length)
{
    SendChar(0,0x33);			/* Put Phantom Button */
    SendChar(1,KeyCode);		/* Send Key Code */
    SendChar(1,((Row<<4)&0x70)|(Col&0xf));/* Send position */
    SendChar(1,Length);			/* Send Key Length */
    return(TestError(NULLP));
}
bool DeleteButton(uchar KeyCode)
{
    SendChar(0,0x34);			/* Delete button code */
    SendChar(1,KeyCode);		/* Send Code to delete */
    while(CheckCPUBUSY());		/* Wait till not busy */
    return(TestError("Button not present to delete"));
}
void DeleteAllButtons(void)
{
    SendChar(0,0x35);			/* Delete button code */
}
uchar ReadKeyCode(void)
{
    uchar c;
    SendChar(0,0x36);
    GetData(&c);
    return(c);
}
void SetButtonAttribute(uchar abc) 
{
    SendChar(0,0x38 + (abc & 0x07));	/* Button Attribute */
}
void SetButtonGrayPalet(uchar Palet) 
{
    SendChar(0,0xd0 + (Palet & 0x0f));	/* Set Button Gray Palet */
}
void SetButtonPlane(uchar Plane)
{
    SendChar(0,0xe0 + (Plane & 0x01));	/* Set Button Plane */
}

/********* Display Control *********/

void BlankDisplay(void)
{
    SendChar(0,0x80);
}
void ClearDisplay(void)
{
    SendChar(0,0x87);
}
void Refresh(void)
{
    SendChar(0,0x81);
}
void SetAutoRefresh(uchar ab)
{
    SendChar(0,0x88+ab);
}
/* DumpDisplayRAM dumps the current display to file named sFileName */
bool DumpDisplayRAM(STRING * sFileName)
{
    uchar j;
    uint i;
    uint k;
    FILE * fpFile;
    k = (ModuleType == TVM2464_TYPE)?1920:15360;
    if((fpFile = fopen(sFileName,"wb")) == NULL)
    {
	printf("ERROR: Opening File %s\n",sFileName);
	return(1);
    }
    SendChar(0,0x84);	/* Dump code */
    for(i=0;i<k;i++)
    {
	if(GetData(&j))
	{
	    printf("ERROR: Dumping Display RAM\n");
	    fclose(fpFile);
	    return(1);
	}
	fputc(j,fpFile);
    }
    fclose(fpFile);
    return(0);
}
bool LoadDisplayRAM(STRING * sFileName)
{
    uchar j;
    uint i;
    uint k;
    FILE * fpFile;
    k = (ModuleType == TVM2464_TYPE)?1920:15360;
    if((fpFile = fopen(sFileName,"rb")) == NULL)
    {
	printf("ERROR: Opening File %s\n",sFileName);
	return(1);
    }
    SendChar(0,0x85);	/* Dump code */
    for(i=0;i<k;i++)
    {
	j = fgetc(fpFile);
	SendChar(1,j);
    }
    fclose(fpFile);
    return(0);
}
void MoveBlockVert(uchar AX,uchar AY,uchar BX,uchar BY,char Distance)
{
    SendChar(0,0x74);
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
    SendChar(1,Distance);
}
void MoveBlockHoriz(uchar AX,uchar AY,uchar BX,uchar BY,char Distance)
{
    SendChar(0,0x75);
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
    SendChar(1,Distance);
}
void LoadGrayDither(uchar GD1,uchar GD2)
{
    SendChar(0,0x82);
    SendChar(1,GD1);
    SendChar(1,GD2);
}
void ReverseVideo(bool bReverse)
{
    SendChar(0,0x8c + (bReverse & 0x01));
}

/******** System Instructions *********/
void SoftReset(void)
{
    SendChar(0,0xfe);
}
void SetContrast(uchar Data)
{
    SendChar(0,0xf3);
    SendChar(1,Data);
}
void SetEL(uchar a)
{
    SendChar(0,0xf4 | (a & 0x01));
}
void NOP(void)
{
    SendChar(0,0xff);
}
void SetBeeper(uchar a)
{
    SendChar(0,0xf0 | (a & 0x01));
}
bool ReadKeyMatrix(uchar * RData)
{
    bool bError=0;
    uint i;
    SendChar(0,0xf2);			/* Send command */
    for(i=0;(i<((ModuleType == TVM2464_TYPE)?6:12))&&!bError;i++)
    {
	bError = GetData(RData);	/* Get data */
	++RData;			/* Incriment pointer */
    }
    if(bError)
    {
	printf("ERROR: Reading key matrix\n");
	return(1);
    }
    return(0);
}
bool ReadAuxPort(uchar * pAuxData)
{
    SendChar(0,0xf7);
    if(GetData(pAuxData))
    {
	return(1);
    }
    return(0);
}
void WriteAuxPort(uchar AuxData)
{
    SendChar(0,0xf8);
    SendChar(1,AuxData);
}
