/************************************************************************/
/*									*/
/*		Copyright 1991, CSys Labs Incorporated,			*/
/*		   	238 So. Hillview Dr.				*/
/*		   	Milpitas, Ca. 95035				*/
/*									*/
/************************************************************************/


// TVIOPP has all methods for the paralell/serial versions of the TVIO routines

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <dos.h>
#include <stdarg.h>
#include <string.h>
#include <iostream.h>
#include <bios.h>
#include <csysdefs.h>
#include <sservpp.h>
#include <tviopp.h>

void Delay(unsigned int time_in_milliseconds)
{
    long Start_Time;
    long Stop_Time;
    Start_Time = biostime(0,0);
    /* Divide time_in_milliseconds by 54.9 ms per tick */
    Stop_Time = Start_Time + (time_in_milliseconds / 55);
    while(Stop_Time >= biostime(0,0));
    return;
}

void tvm::SetModuleType()
{
    uchar x,y;
    SetXY(232,117);
    ReadXY(&x,&y);
    if((x == 232)&&(y == 117))		/* Determin display size */
    {
	MaxX = 239;
	MaxY = 127;
	eModuleType = TVM24128;
	cout << "TVM24128\n";
    }
    else
    {
	MaxX = 239;
	MaxY = 63;
	eModuleType = TVM2464;
	cout << "TVM2464\n";
    }
}

/* StartUp reads a file "base.dat" to see if a BASE address is specified.
If so, it uses this as it's initial assumption of the base address to
try to establish communication with the TVM.  If this file does not exist
or it is in error it first looks at address 0x300 and the 0x307 to try to
establish communication.  After this, it writes the file base.dat so it has
the BASE address next time. StartUp returns 0 if it finds the TVM.  Else
if return 1. */
bool tvmp::StartUp(int iBase)
{
    if(!iBase)
    {
    FILE * fpBase;			/* File pointer to base */
    printf("Opening file base.dat...");
    if((fpBase = fopen("base.dat","rt")) != NULL)
    {
	printf("Reading...");
	if(fscanf(fpBase,"%x",&Base))	/* Get base address */
	{
	    fclose(fpBase);		/* Close file */
	    if(Echo())
	    {
		printf("Complete, BASE = %x\n",Base);
		SetModuleType();	// Set the module type
		return(0);
	    }
	    else
	    {
		printf("\nERROR:Could not establish contact at BASE=%x\n",Base);
	    }
	}
	else
	{
	    fclose(fpBase);		/* Close file */
	    printf("ERROR: Reading file base.dat\n");
	}
    }
    /* POLL TVM board for valid BASE address */
    else
    {
	printf("File not found\n");
    }
    printf("Will POLL for TVM interface...");
    for(Base = 0x300;Base < 0x319;Base +=8)
    {
	printf("%x...",Base);
	if(Echo())
	{
	    printf("Found BASE at %x\n",Base);
	    printf("Creating file base.dat...");
	    if((fpBase = fopen("base.dat","wt")) == NULL)
	    {
		printf("ERROR: Could not create file\n");
	    }
	    else
	    {
		fprintf(fpBase,"%x",Base);
		fclose(fpBase);		/* Close file */
		printf("Complete\n");
	    }
	    SetModuleType();
	    return 0;
	}
    }
    printf("\nCould not establish contact.\n");
    return(1);
    }
    else
    {
	Base = iBase;
	if(Echo())
	{
	    SetModuleType();
	    return(0);		// Return no error
	}
	return 1;               // Return error
    }
}
/* Echo sends a SetXY command to the TVM module.  It then trys to read the
command.  If if does not read the given XY cords, it returns a zero, else it
returns a 1 */
bool tvmp::Echo()
{
    uint i;
    uchar x,y;
    SoftReset();
    Delay(1000);			/* Wait 1 second */
    if(CheckOBF())			/* See if output buffer is full */
    {
	printf("\nOBF\ Valid, Clearing flag\n");
	GetData(&(uchar)i);
    }
    SetXY(100,50);			/* Test to see if TVM is connected */
    ReadXY(&x,&y);
    if((x != 100)||(y != 50))
    {
	return(0);
    }
    SetXY(50,25);			/* Test to see if TVM is connected */
    ReadXY(&x,&y);
    if((x != 50)||(y != 25))
    {
	return(0);
    }
    return(1);
}
/* SetBase sets the Base address for the IO card */
void tvmp::SetBase(uint B)
{
    Base = B;
}
/* GetStatus returns the status register */
uchar tvmp::GetStatus(void)
{
    return(inportb(3+Base));
}

/* CheckIBF tests to see if the input buffer is full.  It return 1 if IBF is
set.  It return 0 if IBF is empty. */
bool tvmp::CheckIBF(void)
{
    if(GetStatus() & 0x01)	/* return IBF flag */
    {
	return(1);
    }
    return(0);
}

/* CheckOBF tests to see if the output buffer is full.  It return 1 if OBF is
full.  It return 0 if OBF is empty. */
bool tvmp::CheckOBF(void)
{
    if(GetStatus() & 0x02)
    {
	return(0);
    }
    return(1);
}

/* CheckERROR tests to see if the Error flag is set.  It return 1 if ERROR is
set.  It return 0 if ERROR is not set. */
bool tvmp::CheckERROR(void)
{
    if(GetStatus() & 0x04)	/* return ERROR flag */
    {
	return(1);
    }
    return(0);
}

/* CheckKEYPRESS tests to see if the KEYPRESS flag is set.  It return 1 if
set.  It return 0 if not set. */
bool tvmp::CheckKEYPRESS(void)
{
    if(GetStatus() & 0x08)	/* return KEYPRESS flag */
    {
	return(1);
    }
    return(0);
}

/* CheckCPUBUSY tests to see if the CPUBUSY flag is set.  It return 1 if
set.  It return 0 if not set. */
bool tvmp::CheckCPUBUSY(void)
{
    if(GetStatus() & 0x10)	/* return CPUBUSY flag */
    {
	return(1);
    }
    return(0);
}

/* SendChar send a character to the TV Module.  It return 0 it sending was
successful.  It returns 1 if error (stayed busy to long.  It gives about
.5 sec to send data. */
bool tvmp::SendChar(uchar uAddr,uchar cChar)
{
    uint i = 0;
    while(CheckIBF())
    {
	if(++i > 50000)
	{
	    printf("ERROR: SendChar timeout\n");
	    return(1);
	}
    }
    outportb(uAddr+Base,cChar); 		/* Send data */
    return(0);
}
/* GetData gets char data from TPDI PPI. */
/* return 0 for successful transfer with valid data in *pData. */
/* If timeout error (data never ready), it returns -1 and data returned is -1.
If ERROR flag set but data was ready it returns 1 with valid data */
bool tvmp::GetData(uchar * pData)
{
    uint i=0;
    while(!CheckOBF())
    {
	if(++i > 50000)			/* Loop timeout, return -1 */
	{
	    printf("ERROR: GetData timeout\n");
	    *pData = 0xff;
	    return(-1);
	}
    }
    *pData = inportb(Base);		/* Get data */
    return(CheckERROR());		/* Return ERROR flag */
}
/* Test error waits till the CPU is not busy.  It then checks the ERROR flag.
If an ERROR has occured, it sends the message to the PC and returns 1.  If no
error it returns 0 */
bool tvmp::TestError(STRING * pString,...)
{
    char Buffer[80];
    va_list argptr;
    while(CheckCPUBUSY());		/* Wait till not busy */
    if(CheckERROR())
    {
	if(pString)
	{
	    va_start(argptr,pString);
	    vsprintf(Buffer,pString,argptr);
	    printf("ERROR: %s\n",Buffer);
	    va_end(argptr);
	}
	return(1);
    }
    return(0);
}

/********** Font Selection **********/
bool tvmp::SelectFont(uchar xxx)
{
    SendChar(0,xxx & 0x7);	/* Send code */
    SendChar(1,0xff);	 	/* Send bogus code to make sure font error
				flag can set if required.  This code will
				be ignored and is used only to delay the
				software untill the SetFont instruction can
				set the flags. */
    return(TestError("NULL Font"));
}
void tvmp::SetFontAttribute(uchar ab)
{
    SendChar(0,0x18 + (ab & 0x3));
}
void tvmp::SetFontGrayPalet(uchar Palet)
{
    SendChar(0,0xb0 + (Palet & 0x0f));	/* Set Font Gray Palet */
}
void tvmp::SetFontPlane(uchar Plane)
{
    SendChar(0,0x1c + (Plane & 0x01));	/* Set Font Palet */
}
/* DownLoadFont loads a font from a file into font memory.  It returns a
0 for a successful load.  Returns a zero if an error. sFileName contains the
name of the file to open. */
bool tvmp::DownLoadFont(STRING * sFileName,uint FontNumber)
{
    FILE * fpFile;
    uchar a;
    if((fpFile = fopen(sFileName,"rb")) == NULL)
    {
	printf("ERROR: File %s not found\n",sFileName);
	return(1);
    }
    SendChar(0,0x08 + (FontNumber & 0x07));	/* Send command */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
	return(1);
    }
    SendChar(1,a);		/* Send Size X */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
	return(1);
    }
    SendChar(1,a);		/* Send Size Y */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
	return(1);
    }
    SendChar(1,a);		/* Send Offset */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
	return(1);
    }
    SendChar(1,a);		/* Send Decenders */
    while(1)
    {
	a = fgetc(fpFile);
	if(feof(fpFile))
	{
	    if(TestError("Down load error"))
	    {
		fclose(fpFile);
		return(1);
	    }
	    fclose(fpFile);
	    return(0);
	}
	if(ferror(fpFile))
	{
	    printf("ERROR: Error reading file %s\n",sFileName);
	    fclose(fpFile);
	    return(1);
	}
	SendChar(1,a);	/* Send FONT data to TPDI */
    }
}
bool tvmp::GetFontChar(FILE * pFile,uchar * pChar)
{
    *pChar = fgetc(pFile);
    if(feof(pFile))
    {
	return(1);
    }
    if(ferror(pFile))
    {
	printf("\nERROR: Error reading file\n");
	return(1);
    }
    return(0);
}

/******** Cursor Positioning ********/
void tvmp::SetXY(uchar XPos,uchar YPos)
{
    SendChar(0,0x20);
    SendChar(1,XPos);
    SendChar(1,YPos);
}
bool tvmp::ReadXY(uchar * XPos,uchar * YPos)
{
    SendChar(0,0x21);
    if(GetData(XPos) || GetData(YPos))
    {
	return(1);
    }
    return(0);
}
void tvmp::CursorUp(void)
{
    SendChar(0,0x22);
}
void tvmp::CursorDown(void)
{
    SendChar(0,0x23);
}
void tvmp::CursorLeft(void)
{
    SendChar(0,0x24);
}
void tvmp::CursorRight(void)
{
    SendChar(0,0x25);
}
void tvmp::SetX(uchar XPos)
{
    SendChar(0,0x26);
    SendChar(1,XPos);
}
void tvmp::SetY(uchar YPos)
{
    SendChar(0,0x27);
    SendChar(1,YPos);
}
void tvmp::CursorAttribute(uchar abc)
{
    SendChar(0,0x10 + (abc & 0x7));
}

/********* Text Configuration *********/
void tvmp::SetTextWindow(uchar AX,uchar AY,uchar BX,uchar BY)
{
    SendChar(0,0x28);	/* Send command  */
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
}
void tvmp::SetPitch(uchar Pitch)
{
    SendChar(0,0x2a);	/* Send Pitch */
    SendChar(1,Pitch);
}
void tvmp::SetHeight(uchar Height)
{
    SendChar(0,0x2b);	/* Send Height */
    SendChar(1,Height);
}

/********* Text Input *********/
/* InputString sends a string to the display */
void tvmp::InputString(STRING * String)
{
    SendChar(0,0x2c);		/* Write string */
    while(*String)		/* Go till NULL terminator */
    {
	if(*String == '\n')
	{

	    SetX(0);
	    CursorDown();
	    SendChar(0,0x2c);		/* Write string */
	}
	else
	{
	    SendChar(1,*String);	/* Send all char's */
	}
	String++;		/* Incriment pointer */
    }
}

/********* Graphics Input **********/
void tvmp::DrawBox(uchar AX,uchar AY,uchar BX,uchar BY,uchar TTTF)
{
    SendChar(0,0x40 + (TTTF & 0x0f));
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
}
void tvmp::DrawBlock(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT)
{
    SendChar(0,0x60 + (TT & 0x03));
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
}
void tvmp::DrawHoriz(uchar XPos,uchar YPos,char Length,uchar TT)
{
    SendChar(0,0x64+(TT&0x03));
    SendChar(1,XPos);
    SendChar(1,YPos);
    SendChar(1,Length);
}
void tvmp::DrawVert(uchar XPos,uchar YPos,char Length,uchar TT)
{
    SendChar(0,0x68+(TT&0x03));
    SendChar(1,XPos);
    SendChar(1,YPos);
    SendChar(1,Length);
}
void tvmp::DrawVector(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT)
{
    SendChar(0,0x6c + (TT & 0x03));
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
}
void tvmp::SetPixel(uchar XPos,uchar YPos,uchar TT)
{
    SendChar(0,0x70+(TT & 0x03));
    SendChar(1,XPos);
    SendChar(1,YPos);
}
void tvmp::SetGrayPalet(uchar Palet)
{
    SendChar(0,0xc0 + (Palet & 0x0f));	/* Set Gray Palet */
}
void tvmp::SetPlane(uchar Plane)
{
    SendChar(0,0x78 + (Plane & 0x01));	/* Set Plane */
}

/******* Button Input *********/

/* PlaceButton return 0 if putton placement was successful.  Returns 1 if error
and button cound not be placed. */
bool tvmp::PlaceButton(uchar Code,uchar Col,uchar Row)
{
    SendChar(0,0x30);		/* Put Button */
    SendChar(1,Code);		/* Send Key Code */
    SendChar(1,((Row<<4)&0x70)|(Col&0xf));	/* Send position */
    return(TestError(NULLP));
}
void tvmp::LoadButtonBuffer(STRING * String)
{
    register STRING * pS = String;
    SendChar(0,0x31);		/* Send Button String command */
    while(*pS)
    {
	SendChar(1,*pS++);
    }
}
/* Return 0 if no error, 1 if error */
bool tvmp::GetButtonSize(uchar KeyCode,uchar * RData)
{
    SendChar(0,0x32);			/* Send OP */
    SendChar(1,KeyCode);		/* Send Code */
    return(GetData(RData));		/* Get Data */
}
bool tvmp::PlacePhantomButton(uchar KeyCode,uchar Col,uchar Row,uchar Length)
{
    SendChar(0,0x33);			/* Put Phantom Button */
    SendChar(1,KeyCode);		/* Send Key Code */
    SendChar(1,((Row<<4)&0x70)|(Col&0xf));/* Send position */
    SendChar(1,Length);			/* Send Key Length */
    return(TestError(NULLP));
}
bool tvmp::DeleteButton(uchar KeyCode)
{
    SendChar(0,0x34);			/* Delete button code */
    SendChar(1,KeyCode);		/* Send Code to delete */
    while(CheckCPUBUSY());		/* Wait till not busy */
    return(TestError("Button not present to delete"));
}
void tvmp::DeleteAllButtons(void)
{
    SendChar(0,0x35);			/* Delete button code */
}
uchar tvmp::ReadKeyCode(void)
{
    uchar c;
    SendChar(0,0x36);
    GetData(&c);
    return(c);
}
void tvmp::SetButtonAttribute(uchar abc)
{
    SendChar(0,0x38 + (abc & 0x07));	/* Button Attribute */
}
void tvmp::SetButtonGrayPalet(uchar Palet)
{
    SendChar(0,0xd0 + (Palet & 0x0f));	/* Set Button Gray Palet */
}
void tvmp::SetButtonPlane(uchar Plane)
{
    SendChar(0,0xe0 + (Plane & 0x01));	/* Set Button Plane */
}

/********* Display Control *********/

void tvmp::BlankDisplay(void)
{
    SendChar(0,0x80);
}
void tvmp::ClearDisplay(void)
{
    SendChar(0,0x87);
}
void tvmp::Refresh(void)
{
    SendChar(0,0x81);
}
void tvmp::SetAutoRefresh(uchar ab)
{
    SendChar(0,0x88+ab);
}
/* DumpDisplayRAM dumps the current display to file named sFileName */
bool tvmp::DumpDisplayRAM(STRING * sFileName)
{
    uchar j;
    uint i;
    FILE * fpFile;
    if((fpFile = fopen(sFileName,"wb")) == NULL)
    {
	printf("ERROR: Opening File %s\n",sFileName);
	return(1);
    }
    SendChar(0,0x84);	/* Dump code */
    for(i=0;i<(eModuleType == TVM2464?1920:15360);i++)
    {
	if(GetData(&j))
	{
	    printf("ERROR: Dumping Display RAM\n");
	    fclose(fpFile);
	    return(1);
	}
	fputc(j,fpFile);
    }
    fclose(fpFile);
    return(0);
}
bool tvmp::LoadDisplayRAM(STRING * sFileName)
{
    uchar j;
    uint i;
    FILE * fpFile;
    if((fpFile = fopen(sFileName,"rb")) == NULL)
    {
	printf("ERROR: Opening File %s\n",sFileName);
	return(1);
    }
    SendChar(0,0x85);	/* Dump code */
    for(i=0;i<(eModuleType == TVM2464?1920:15360);i++)
    {
        j = fgetc(fpFile);
	SendChar(1,j);
    }
    fclose(fpFile);
    return(0);
}
void tvmp::MoveBlockVert(uchar AX,uchar AY,uchar BX,uchar BY,char Distance)
{
    SendChar(0,0x74);
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
    SendChar(1,Distance);
}
void tvmp::MoveBlockHoriz(uchar AX,uchar AY,uchar BX,uchar BY,char Distance)
{
    SendChar(0,0x75);
    SendChar(1,AX);
    SendChar(1,AY);
    SendChar(1,BX);
    SendChar(1,BY);
    SendChar(1,Distance);
}
void tvmp::LoadGrayDither(uchar GD1,uchar GD2)
{
    SendChar(0,0x82);
    SendChar(1,GD1);
    SendChar(1,GD2);
}
void tvmp::ReverseVideo(uchar bReverse)
{
    SendChar(0,0x8c+(bReverse & 0x1));
}

/******** System Instructions *********/
void tvmp::SoftReset(void)
{
    SendChar(0,0xfe);
}
void tvmp::SetContrast(uchar Data)
{
    SendChar(0,0xf3);
    SendChar(1,Data);
}
void tvmp::SetEL(uchar a)
{
    SendChar(0,0xf4 | (a & 0x01));
}
void tvmp::NOP()
{
    SendChar(0,0xff);
}
void tvmp::SetBeeper(uchar a)
{
    SendChar(0,0xf0 | (a & 0x01));
}
bool tvmp::ReadKeyMatrix(uchar * RData)
{
    bool bError=0;
    uint i;
    SendChar(0,0xf2);			/* Send command */
    for(i=0;(i<(eModuleType == TVM2464?6:12))&&!bError;i++)
    {
	bError = GetData(RData);	/* Get data */
	++RData;			/* Incriment pointer */
    }
    if(bError)
    {
	printf("ERROR: Reading key matrix\n");
	return(1);
    }
    return(0);
}
bool tvmp::ReadAuxPort(uchar * pAuxData)
{
    SendChar(0,0xf7);
    if(GetData(pAuxData))
    {
	return(1);
    }
    return(0);
}
void tvmp::WriteAuxPort(uchar AuxData)
{
    SendChar(0,0xf8);
    SendChar(1,AuxData);
}


/************************** tvios methods ********************************/

static STRING BuffA[200];	/* Buffer used for TX communication */
// extern bool bTXBusy;
// extern bool bKPS;		/* KPS signal */
/*-------------------------------------------------------------------------*/
/*These routines are the low level communications for the serial link. */

tvms::tvms() : sserver(this)
{
	yystate = -1;
	sMark = 0;
	Next = 0;
	Verbose = 0;
	eModuleType = TVM2464;
};

tvms::~tvms()
{
    CloseServer();
};

/* ssprintf sends the contents of sBuff out to the serial interface */
void tvms::ssprintf(STRING * sBuff,STRING * pFormat,...)
{
    va_list argptr;
    va_start(argptr,pFormat);
    while(bTXBusy);
    vsprintf(sBuff,pFormat,argptr);
    SendString(sBuff);
    if(Verbose)printf("%s",sBuff);
    va_end(argptr);
}

/* yylex does the lexical analisis of the data returned from the TVM.  It
returns a token specifying the return type and puts the lexeme into the
buffer "text" (actual string data returned is the lexeme).  A space, \n or ,
mark the end of the lexeme.

	Lexeme		Tokenvalue returned	Token definition
	______		___________________	______
	$<number>	0	                NUMERIC
	#<cmd string>>	1                       ERROR
	@E<string>      2			ECHO
	@K<bool>	3			KEYPRESSED
	&<number>	4			KEYCODE
	NULL TOKEN	-1			NTOK

*/

int tvms::yylex()
{
    char yynext;		/* Storage for next character */
    uint yytoken;		/* Temp token storage */
    STRING * pText;		/* Pointer to lex text buffer */
    uint i;
    uint index;			/* Temporary indexer */
    if(RBIn - Next <= 0)return NTOK;	/* No data available */
    yynext = RBuffer[Next & RBMASK];	/* Get next data */
#ifdef NODEF
    printf("<%c>",yynext);
#endif
    /* Get yystate */
    if(yystate == -1)		/* No accept state yet */
    {
	switch(yynext)
	{
	    case '$':		/* Found NUMERIC */
		yystate = NUMERIC;
	        break;
	    case '#':		/* Found ERROR */
		yystate = ERROR;
	        break;
	    case '@':		/* Found serial interface command */
		yystate = SIC;
		break;
	    case '&':		/* Found KEYPRESSED */
		yystate = KEYPRESSED;
		break;
	    default:
		sMark = ++Next;	/* Advance past character */
		return NTOK;
	}
    }
    else if(yystate == SIC)	/* SIC State */
    {
	if(yynext == 'K')yystate = KEYPRESSED;
	else if(yynext == 'E')yystate = ECHO;
	else
	{
	    sMark = Next;	       /* Move sMark past character */
	    yystate = -1;
            return NTOK;
	}
    }
    else				/* Had accept state */
    {
	switch(yynext)
        {
	    case ' ':            	/* Found terminator, transfer lexeme */
	    case '\n':
	    case ',':
                yytoken = yystate;	/* Save state for token */
		yystate = -1;		/* Reset state */
		pText = text;		/* Set pointer */
		index = sMark;		/* Set index to get data */
		for(i=0;i<(Next - sMark);i++)/* Transfer lexeme */
		{
		    *pText++ = RBuffer[index++ & RBMASK];
		}
		*pText = 0;		/* Set null terminator */
		sMark = ++Next;		/* Advance indexers */
		if(Verbose)printf("[%s]",text);
		return yytoken;		/* Return token value */
	}
    }
    Next++;
    return NTOK;
}

/* Echo sends an echo command.  It then waits for the echo
to come back.  If something is returned before the echo, it returns this
token.  Lexeme is returned in text2 in case echo was precedded by something.
If the echo is returned, it returns ECHO token.   Echo
automatically selects a unique echo string to send.  It changes
this string each time.  */

uint tvms::Echo()
{
    STRING EchoS[30];
    STRING temp[20];
    static uint Numb = 0;
    int i = -1;
    int tok = -1;
    /* Get a random number in a string */
    sprintf(EchoS,"@E%02d",Numb++%100);	/* Form command and send */
    ssprintf(BuffA,"%s\n",EchoS);	/* Send string with \n at end */
    while(1)
    {
	while((i = yylex()) == NTOK)	/* Get token */
	{
	}
	if(tok == -1)
	{
	    strcpy(text2,text);		/* Copy lexeme value into text2 */
	    tok = i;			/* Initial assignment */
	}
	if(i == ECHO)			/* Loop untill ECHO is a exact match */
	{
	    if(!strcmp(text,EchoS))return tok;/* Return previous tok value */
	}
	else
	{
	    tok = i;			/* Save token value */
	    strcpy(text2,text);		/* Copy lexeme value into text2 */
	}
    }
}

/* SetLowThreshold sets the module low threshold point */
void tvms::SetLowThreshold(uint Threshold)
{
    ssprintf(BuffA,"@L%d\n",Threshold);
}

/* SetHighThreshold sets the module low threshold point */
void tvms::SetHighThreshold(uint Threshold)
{
    ssprintf(BuffA,"@H%d\n",Threshold);
}

/* EnableXON enables the XON/XOFF software hand shaking */
void tvms::EnableXON()
{
    ssprintf(BuffA,"@X\n");
}

/* ResetXON resets the XON/XOFF software hand shaking */
void tvms::ResetXON()
{
    ssprintf(BuffA,"@R\n");
}
/*-------------------------------------------------------------------------*/

/* CheckKEYPRESS tests to see if a key has been pressed.  It return 1 if 
set.  It return 0 if not set. */
bool tvms::CheckKEYPRESS(void)
{
    if(bKPS)
    {
	bKPS = 0;
        return 1;
    }
    return 0;
#ifdef NODEF
    STRING * pS;
    int i;
    ssprintf(BuffA,"\n@K ");
    while(yylex() != KEYPRESSED);	/* Wait for valid token */
    return atoi(&text[2]);
#endif
}
/* GetData gets char data from TVM serial port. */
/* return 0 for successful transfer with valid data in *pData.  If error
is detected, it returns a 1 but still waits for a number. */
bool tvms::GetData(uchar * pData)
{
    STRING * pS;
    int i;
    bool bError = 0;
    while(1)
    {
	while((i = yylex()) == NTOK);	/* Wait for valid token */
	if(i == ERROR)bError = 1;	/* Error so set flag */
	if(i == NUMERIC)
	{
	    pS = &text[1];		/* Set pointer */
    	    *pData = atoi(pS);
    	    return bError;
        }
    }
}
/* StartUp initializes the COM port for communication. */
bool tvms::StartUp(int Port,int IRQ,uint iBaud,STRING * Protocol)
{
    OpenServer(Port,IRQ,iBaud,Protocol);/* Open COM server */
    ssprintf(BuffA,"@S\n");		/* Enable KPS Signal */
    SetModuleType();			// Set the module type
    return(0);
}
/* Test error waits till the CPU is not busy.  It then checks the ERROR flag.
If an ERROR has occured, it sends the message to the PC and returns 1.  If no
error it returns 0 */
bool tvms::TestError(STRING * pString,...)
{
    char Buffer[80];
    va_list argptr;
    if(Echo() == ERROR)		/* Send echo and see if an error */
    {
	if(pString)
	{
    	    va_start(argptr,pString);
	    vsprintf(Buffer,pString,argptr);
	    printf("ERROR: %s\n",Buffer);
	    va_end(argptr);
	}
	return(1);
    }
    return(0);
}

/********** Font Selection **********/
bool tvms::SelectFont(uchar xxx)
{
    ssprintf(BuffA,"\nFS%d ",xxx & 0x7);	/* Send string */
    return(TestError("NULL Font"));
}
void tvms::SetFontAttribute(uchar ab)
{
    ssprintf(BuffA,"\nFA%d ",ab & 0x3);	/* Send string */
}
void tvms::SetFontGrayPalet(uchar Palet)
{
    ssprintf(BuffA,"\nFG%d ",Palet & 0xf);	/* Send string */
}
void tvms::SetFontPlane(uchar Plane)
{
    ssprintf(BuffA,"\nFP%d ",Plane & 0x1);	/* Send string */
}
/* DownLoadFont loads a font from a file into font memory.  It returns a
0 for a successful load.  Returns a zero if an error. sFileName contains the
name of the file to open. */
bool tvms::DownLoadFont(STRING * sFileName,uint FontNumber)
{
    FILE * fpFile;
    uchar a;
    uint ModCount = 0;
    uchar Buffer[256];
    uchar * pB;
    bool bEOF = 0;
    bool bError = 0;
    if((fpFile = fopen(sFileName,"rb")) == NULL)
    {
	printf("ERROR: File %s not found\n",sFileName);
	return(1);
    }
    ssprintf(BuffA,"\nFD%d,",(FontNumber & 0x7));
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
	return(1);
    }
    ssprintf(BuffA,"%d,",a);	/* Send size X */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
	return(1);
    }
    ssprintf(BuffA,"%d,",a);	/* Send size Y */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
	return(1);
    }
    ssprintf(BuffA,"%d,",a);	/* Send offset */
    if(GetFontChar(fpFile,&a))
    {
	fclose(fpFile);
	return(1);
    }
    ssprintf(BuffA,"%d,",a);	/* Send decenders */
    if(TestError("Down load error"))	/* Check for errors */
    {
	fclose(fpFile);
	return 1;
    }
    while(!bEOF)		/* No error so down load */
    {
	ssprintf(BuffA,"!");	/* Send block transfer character */
	pB = Buffer;		/* Set pointer */
	do	/* Get block of 256 bytes */
	{
	    if(!bEOF)
	    {
		a = fgetc(fpFile);
		if(feof(fpFile))
		{
		    fclose(fpFile);
		    bEOF = 1;
		    a = 0;
		}
		else if(ferror(fpFile))
		{
		    printf("ERROR: Error reading file %s\n",sFileName);
		    fclose(fpFile);
		    bEOF = 1;
		    bError = 1;
		    a = 0;
		}
		*pB++ = a;
	    }
	    else
	    {
		*pB++ = 0;
	    }
	}while(++ModCount % 256);
	SendBlock((uchar *)Buffer,256);	/* Send block of 256 bytes */
	while(!bTXBusy);	/* Wait till not busy to protect buffer */
    }
    return bError;
}
bool tvms::GetFontChar(FILE * pFile,uchar * pChar)
{
    *pChar = fgetc(pFile);
    if(feof(pFile))
    {
	return(1);
    }
    if(ferror(pFile))
    {
	printf("\nERROR: Error reading file\n");
        return(1);
    }
    return(0);
}

/******** Cursor Positioning ********/
void tvms::SetXY(uchar XPos,uchar YPos)
{
    ssprintf(BuffA,"\nCS%d,%d ",XPos,YPos);
}
bool tvms::ReadXY(uchar * XPos,uchar * YPos)
{
    ssprintf(BuffA,"\nCG ");
    if(GetData(XPos) || GetData(YPos))
    {
	return(1);
    }
    return(0);
}
void tvms::CursorUp(void)
{
    ssprintf(BuffA,"\nCU ");
}
void tvms::CursorDown(void)
{
    ssprintf(BuffA,"\nCD ");
}
void tvms::CursorLeft(void)
{
    ssprintf(BuffA,"\nCL ");
}
void tvms::CursorRight(void)
{
    ssprintf(BuffA,"\nCR ");
}
void tvms::SetX(uchar XPos)
{
    ssprintf(BuffA,"\nCX%d ",XPos);
}
void tvms::SetY(uchar YPos)
{
    ssprintf(BuffA,"\nCY%d,%d ",YPos);
}
void tvms::CursorAttribute(uchar abc)
{
    ssprintf(BuffA,"\nCA%d ",abc & 0x7);
}

/********* Text Configuration *********/
void tvms::SetTextWindow(uchar AX,uchar AY,uchar BX,uchar BY)
{
    ssprintf(BuffA,"\nTW%d,%d,%d,%d ",AX,AY,BX,BY);
}
void tvms::SetPitch(uchar Pitch)
{
    ssprintf(BuffA,"\nTP%d ",Pitch);
}
void tvms::SetHeight(uchar Height)
{
    ssprintf(BuffA,"\nTH%d ",Height);
}
/********* Text Input *********/
/* InputString sends a string to the display */
void tvms::InputString(STRING * String)
{
    char * cBuff = new char[strlen(String)+1];
    STRING * pS;
    pS = cBuff;
    *pS = 0;
    while(*String)		/* Go till NULL terminator */
    {
	if(*String == '\n')
	{
	    if(strlen(cBuff))
	    {
		ssprintf(BuffA,"\nIS\"%s\"",cBuff);
		pS = cBuff;
		*pS = 0;
	    }
	    SetX(0);
	    CursorDown();
	}
	else
	{
	    *pS++ = *String;
	    *pS = 0;
	}
	String++;		/* Move pointer */
    }
    if(strlen(cBuff))
    {
	ssprintf(BuffA,"\nIS\"%s\" ",cBuff);
    }
    delete cBuff;
}
/********* Graphics Input **********/
void tvms::DrawBox(uchar AX,uchar AY,uchar BX,uchar BY,uchar TTTF)
{
    ssprintf(BuffA,"\nGX%d,%d,%d,%d,%d ",TTTF & 0xf,AX,AY,BX,BY);
}
void tvms::DrawBlock(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT)
{
    ssprintf(BuffA,"\nGB%d,%d,%d,%d,%d ",TT & 0x3,AX,AY,BX,BY);
}
void tvms::DrawHoriz(uchar XPos,uchar YPos,char Length,uchar TT)
{
    ssprintf(BuffA,"\nGH%d,%d,%d,%d ",TT & 0x3,XPos,YPos,Length,TT&0x3);
}
void tvms::DrawVert(uchar XPos,uchar YPos,char Length,uchar TT)
{
    ssprintf(BuffA,"\nGV%d,%d,%d,%d ",TT & 0x3,XPos,YPos,Length,TT&0x3);
}
void tvms::DrawVector(uchar AX,uchar AY,uchar BX,uchar BY,uchar TT)
{
    ssprintf(BuffA,"\nGC%d,%d,%d,%d,%d ",TT & 0x3,AX,AY,BX,BY);
}
void tvms::SetPixel(uchar XPos,uchar YPos,uchar TT)
{
    ssprintf(BuffA,"\nGI%d,%d,%d,%d ",TT & 0x3,XPos,YPos,TT&0x3);
}
void tvms::SetGrayPalet(uchar Palet)
{
    ssprintf(BuffA,"\nGG%d ",Palet & 0xf);
}
void tvms::SetPlane(uchar Plane)
{
    ssprintf(BuffA,"\nGP%d ",Plane & 0x1);
}

/******* Button Input *********/

/* PlaceButton return 0 if putton placement was successful.  Returns 1 if error
and button cound not be placed. */
bool tvms::PlaceButton(uchar Code,uchar Col,uchar Row)
{
    ssprintf(BuffA,"\nBC%d,%d",Code,((Row<<4)&0x70)|(Col&0xf));
    return(TestError(NULLP));
}
void tvms::LoadButtonBuffer(STRING * String)
{
    ssprintf(BuffA,"\nBL\"%s\" ",String);
}
/* Return 0 if no error, 1 if error */
bool tvms::GetButtonSize(uchar KeyCode,uchar * RData)
{
    ssprintf(BuffA,"\nBS%d ",KeyCode);
    return(GetData(RData));		/* Get Data */
}
bool tvms::PlacePhantomButton(uchar KeyCode,uchar Col,uchar Row,uchar Length)
{
    ssprintf(BuffA,"\nBH%d,%d,%d ",KeyCode,(((Row<<4)&0x70)|(Col&0xf)),Length);
    return(TestError(NULLP));
}
bool tvms::DeleteButton(uchar KeyCode)
{
    ssprintf(BuffA,"\nBD%d ",KeyCode);
    return(TestError("Button not present to delete"));
}
void tvms::DeleteAllButtons(void)
{
    ssprintf(BuffA,"\nBE ");
}
uchar tvms::ReadKeyCode(void)
{
    uchar c;
    ssprintf(BuffA,"\nBK ");
    GetData(&c);
    return(c);
}
void tvms::SetButtonAttribute(uchar abc)
{
    ssprintf(BuffA,"\nBA%d ",(abc & 0x07));
}
void tvms::SetButtonGrayPalet(uchar Palet)
{
    ssprintf(BuffA,"\nBA%d ",(Palet & 0x0f));
}
void tvms::SetButtonPlane(uchar Plane)
{
    ssprintf(BuffA,"\nBA%d ",(Plane & 0x01));
}

/********* Display Control *********/

void tvms::BlankDisplay(void)
{
    ssprintf(BuffA,"\nDB ");
}
void tvms::ClearDisplay(void)
{
    ssprintf(BuffA,"\nDC ");
}
void tvms::Refresh(void)
{
    return;
}
void tvms::SetAutoRefresh(uchar)
{
    return;
}
/* DumpDisplayRAM dumps the current display to file named sFileName */
bool tvms::DumpDisplayRAM(STRING *)
{
    return(0);
}
bool tvms::LoadDisplayRAM(STRING *)
{
    return(0);
}
void tvms::MoveBlockVert(uchar AX,uchar AY,uchar BX,uchar BY,char Distance)
{
    ssprintf(BuffA,"\nDV%d,%d,%d,%d,%d ",AX,AY,BX,BY,Distance);
}
void tvms::MoveBlockHoriz(uchar AX,uchar AY,uchar BX,uchar BY,char Distance)
{
    ssprintf(BuffA,"\nDH%d,%d,%d,%d,%d ",AX,AY,BX,BY,Distance);
}
void tvms::LoadGrayDither(uchar GD1,uchar GD2)
{
    ssprintf(BuffA,"\nDD%d,%d ",GD1,GD2);
}
void tvms::ReverseVideo(uchar bReverse)
{
    ssprintf(BuffA,"\nDR%d ",bReverse);
}

/******** System Instructions *********/
void tvms::SoftReset(void)
{
    ssprintf(BuffA,"\nSR ");
}
void tvms::SetContrast(uchar Data)
{
    ssprintf(BuffA,"\nSC%d ",Data);
}
void tvms::SetEL(uchar a)
{
    ssprintf(BuffA,"\nSE%d ",a & 0x01);
}
void tvms::NOP(void)
{
    ssprintf(BuffA,"\nSN ");
}
void tvms::SetBeeper(uchar a)
{
    ssprintf(BuffA,"\nSB%d ",a & 0x01);
}
bool tvms::ReadKeyMatrix(uchar * RData)
{
    bool bError=0;
    uint i;
    ssprintf(BuffA,"\nSK ");
    for(i=0;(i<12)&&!bError;i++)
    {
	bError = GetData(RData);	/* Get data */
	++RData;			/* Incriment pointer */
    }
    if(bError)
    {
	printf("ERROR: Reading key matrix\n");
	return(1);
    }
    return(0);
}
bool tvms::ReadAuxPort(uchar * pAuxData)
{
    ssprintf(BuffA,"\nSA ");
    if(GetData(pAuxData))
    {
	return(1);
    }
    return(0);
}
void tvms::WriteAuxPort(uchar AuxData)
{
    ssprintf(BuffA,"\nSW%d ",AuxData);
}
